<?php

require_once(dirname(__FILE__).'/const.php');
require_once(dirname(__FILE__).'/migrator_private_categories.php');
require_once(dirname(__FILE__).'/migrator_post_levels.php');
require_once(dirname(__FILE__).'/migrator_post_restrictions.php');
require_once(dirname(__FILE__).'/migrator_view_levels.php');

// ----------------------------------------------------------------------------
// the DiscloseSecret_Migrator class can migrate various plugins to
// DiscloseSecret; also used to migrate its predecessor
// ----------------------------------------------------------------------------

class DiscloseSecret_Migrator
{
	var $m_migrators = array();

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	function DiscloseSecret_Migrator()
	{
		$migrator =& new DiscloseSecret_MigratorPrivateCategories();
		$this->m_migrators[$migrator->get_name()] =& $migrator;

		$migrator =& new DiscloseSecret_MigratorPostLevels();
		$this->m_migrators[$migrator->get_name()] =& $migrator;

		$migrator =& new DiscloseSecret_MigratorPostRestrictions();
		$this->m_migrators[$migrator->get_name()] =& $migrator;

		$migrator =& new DiscloseSecret_MigratorViewLevels();
		$this->m_migrators[$migrator->get_name()] =& $migrator;
	}

	// ----------------------------------------------------------------------------
	// migration functions
	// ----------------------------------------------------------------------------

	function enforce_constraints()
	{
		// just do some clean up to be sure our metas are consistent;
		// after migration it is mandatory but we can do it everywhere
		global $wpdb;

		// selecting duplicate restrictions with equal value
		$query = "
			SELECT post_id, meta_key, meta_value, count(*) AS count
			FROM {$wpdb->postmeta}
			WHERE meta_key IN ('". DiscloseSecret_META_CAP. "'
				, '". DiscloseSecret_META_ROLE. "'
				, '". DiscloseSecret_META_USER. "')
			GROUP BY post_id, meta_key, meta_value
			HAVING count > 1
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		// deleting duplicates with equal value
		foreach ($s as $i)
		{
			$query = "
				DELETE
				FROM {$wpdb->postmeta}
				WHERE post_id = {$i->post_id}
				AND meta_key = '{$i->meta_key}'
				AND meta_value = '{$i->meta_value}'
				LIMIT ". strval($i->count - 1). "
			";
			$wpdb->query($query);
		}
	}

	function migrate($interactive = true, $plugin_name = NULL)
	{
		if (isset($plugin_name))
		{
			$status = $this->m_migrators[$plugin_name]->migrate($interactive);
			return $status == DiscloseSecret_MIGRATION_DONE;
		}

		$migrated = false;
		foreach (array_keys($this->m_migrators) as $key)
		{
			$status = $this->m_migrators[$key]->migrate($interactive);
			if ($status == DiscloseSecret_MIGRATION_DONE)
			{
				$migrated = true;
				$this->enforce_constraints();
			}
		}
		return $migrated;
	}

	function print_form()
	{
		foreach (array_keys($this->m_migrators) as $key)
		{
			$this->m_migrators[$key]->print_form();
		}
	}
}

?>