<?php

require_once(dirname(__FILE__).'/const.php');

define('DiscloseSecret_ACTION_MIGRATE_VIEWLEVELS', 'migrate_view_levels');
define('DiscloseSecret_CONFIG_FORM_ID_VIEWLEVELS', 'view_levels_migrate');

define('DiscloseSecret_VIEWLEVELS_META', 'viewlevel');
define('DiscloseSecret_VIEWLEVELS_OPTION_DEFAULT', 'vl_default');

// ----------------------------------------------------------------------------
// the DiscloseSecret_MigratorViewLevels class migrates the View Levels plugin
// to Disclose-Secret
// ----------------------------------------------------------------------------

class DiscloseSecret_MigratorViewLevels
{
	var $m_status;

	function DiscloseSecret_MigratorViewLevels()
	{
		$this->m_status = DiscloseSecret_MIGRATION_NOT_NEEDED;
	}

	function get_name()
	{
		return 'MigratorViewLevels';
	}

	function migrate($interactive = true)
	{
		$this->m_status = $this->migrate_plugin($interactive);
		return $this->m_status;
	}

	function migrate_plugin($interactive = true)
	{
		global $wpdb;

		if ($interactive)
			$do_migration = isset($_POST[DiscloseSecret_ACTION_MIGRATE_VIEWLEVELS]);
		else
			$do_migration = true;

		// get the plugin options
		$viewlevels_post_meta_keys[] = DiscloseSecret_VIEWLEVELS_META;
		$viewlevels_post_meta_in = "'". implode("', '", $viewlevels_post_meta_keys). "'";
		$viewlevels_default = get_option(DiscloseSecret_VIEWLEVELS_OPTION_DEFAULT);
		if ($viewlevels_default === false)
			$viewlevels_default = -1;

		// find out whether plugin is installed
		$viewlevels_installed = false;
		$query = "
			SELECT COUNT(*)
			FROM {$wpdb->postmeta}
			WHERE meta_key IN ($viewlevels_post_meta_in)
			AND meta_value IN (-1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		if ($wpdb->get_var($query) > 0)
			$viewlevels_installed = true;

		if (!$viewlevels_installed)
			return DiscloseSecret_MIGRATION_NOT_NEEDED;
		if (!$do_migration)
			return DiscloseSecret_MIGRATION_SKIPPED;
		if ($interactive)
			check_admin_referer(DiscloseSecret_ACTION_MIGRATE_VIEWLEVELS);

		// do migration if requested
		// delete nonfunctional view levels settings
		$query = "
			SELECT ID as post_id
			FROM {$wpdb->posts}
			WHERE post_status NOT IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$unchanged = array();
		foreach ($s as $i)
		{
			$unchanged[] = $i->post_id;
			$query = "
				DELETE
				FROM {$wpdb->postmeta}
				WHERE meta_key IN ($viewlevels_post_meta_in)
				AND post_id = {$i->post_id}
			";
			$wpdb->query($query);
		}
		$unchanged_in = "'". implode("', '", $unchanged). "'";

		// migrate explicitly set protected posts
		$query = "
			SELECT DISTINCT post_id
			FROM {$wpdb->postmeta}
			WHERE meta_key IN ($viewlevels_post_meta_in)
			AND meta_value IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$protected = array();
		foreach ($s as $i)
		{
			$protected[] = $i->post_id;
			$query = "
				UPDATE {$wpdb->posts}
				SET post_status = 'private'
				WHERE post_status IN ('publish')
				AND ID = {$i->post_id}
			";
			$wpdb->query($query);
		}
		$protected_in = "'". implode("', '", $protected). "'";

		$query = "
			UPDATE {$wpdb->postmeta}
			SET meta_key = '". DiscloseSecret_META_CAP. "',
				meta_value = CONCAT('". DiscloseSecret_CAPABILITY_PREFIX_LEVEL. "', meta_value)
			WHERE meta_key IN ($viewlevels_post_meta_in)
			AND meta_value IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$wpdb->query($query);


		// migrate explicitly set public posts
		$query = "
			SELECT DISTINCT post_id
			FROM {$wpdb->postmeta}
			WHERE meta_key IN ($viewlevels_post_meta_in)
			AND meta_value IN (-1)
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$publish = array();
		foreach ($s as $i)
		{
			// there was also a user level set to restrict access
			// to this post, so we do not make it publish here
			if (in_array($i->post_id, $protected))
				continue;
			$publish[] = $i->post_id;
		}
		$publish_in = "'". implode("', '", $publish). "'";

		$query = "
			DELETE
			FROM {$wpdb->postmeta}
			WHERE meta_key IN ($viewlevels_post_meta_in)
			AND meta_value = -1;
		";
		$wpdb->query($query);

		if ($viewlevels_default > -1)
		{
			// migrate default view level
			$query = "
				UPDATE {$wpdb->posts}
				SET post_status = 'private'
				WHERE post_status IN ('publish')
				AND ID NOT IN ($unchanged_in)
				AND ID NOT IN ($publish_in)
				AND ID NOT IN ($protected_in)
			";
			$wpdb->query($query);

			$query = "
				SELECT ID AS post_id
				FROM {$wpdb->posts}
				WHERE ID NOT IN ($unchanged_in)
				AND ID NOT IN ($publish_in)
				AND ID NOT IN ($protected_in)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			foreach ($s as $i)
			{
				$query = "
					INSERT INTO {$wpdb->postmeta}
					(post_id, meta_key, meta_value)
					VALUES ({$i->post_id}, '". DiscloseSecret_META_CAP. "'
						, '". DiscloseSecret_CAPABILITY_PREFIX_LEVEL. "$viewlevels_default')
				";
				$wpdb->query($query);
			}
		}

		// delete obsolete view levels postmetas
		$query = "
			DELETE
			FROM {$wpdb->postmeta}
			WHERE meta_key IN ($viewlevels_post_meta_in)
		";
		$wpdb->query($query);

		return DiscloseSecret_MIGRATION_DONE;
	}

	function print_form()
	{
		if ($this->m_status == DiscloseSecret_MIGRATION_SKIPPED)
		{
?>
	<div class="wrap">
		<h2><?php _e('Migration from View Levels', DiscloseSecret_PLUGIN_ID); ?></h2>
		<p><?php _e('Disclose-Secret has found that your blog used a version of the <a href="http://blog.firetree.net/2005/08/25/viewlevel-20/">View Levels</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your view levels to Disclose-Secret by pressing the button below.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Summary: After migration your blog\'s already published posts are converted to be protected by Disclose-Secret according to your previous View Levels configuration. Generally Disclose-Secret is more restrictive to your protected posts. E.g. they will not be published at an update service. Remember that there is no default restriction applied to new posts anymore. You also should consider to install a Role Manager. See the common plugin repositories for implementations.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Migration was tested with ViewLevel2 version 2.0.0 but should work with all previous versions including ViewLevel.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><strong><?php _e('Note, that migration can not be undone by this plugin once you have pressed the button below.', DiscloseSecret_PLUGIN_ID); ?></strong></p>
		<form action="" method="post" id="<?php echo DiscloseSecret_CONFIG_FORM_ID_VIEWLEVELS; ?>">
			<p class="submit">
				<input type="submit" name="<?php echo DiscloseSecret_ACTION_MIGRATE_PRIVATE_VIEW_LEVELS; ?>" value="<?php _e('Migrate View Levels &raquo;', DiscloseSecret_PLUGIN_ID); ?>" />
				<?php wp_nonce_field(DiscloseSecret_ACTION_MIGRATE_PRIVATE_VIEW_LEVELS); ?>

			</p>
		</form>
	</div>
<?php
		}
	}
}

?>