<?php

require_once(dirname(__FILE__).'/const.php');

define('DiscloseSecret_PARSE_NOT_FINISHED', -1);
define('DiscloseSecret_PARSE_INCOMPLETE', 0);
define('DiscloseSecret_PARSE_COMPLETE', 1);
define('DiscloseSecret_PARSE_SKIP_CALLBACK', 2);

// ----------------------------------------------------------------------------
// PCRE replacement because processing the pattern proved to be a) not working
// in PCRE 6.6 b) unreliable in PCRE 6.7. Sick!
// ((\'\")((?:[^\\2\\\]|\\\.)*?)\\2)|(?<!SET)?[^\w-]*(([\w-]+?\.)?(<column_name>)\s*(?:(=|!=|<>|<|<=|>|>=)\s*(([\'\"])(?:[^\\8\\\]|\\\.)*?\\8)|\s+(in)\s*(\([^\)]*?\))))\s*(?!,)?
// ----------------------------------------------------------------------------

class DiscloseSecret_PCRE
{
	function parse_strcasecmp($needle, $haystack, $offset)
	{
		$length = strlen($needle);
		if ($length == 1)
			return $needle[0] == $haystack[$offset];
		else
			return !strcasecmp($needle, substr($haystack, $offset, $length));
	}

	function parse_casecmp($needle, $haystack, $offset, &$index)
	{
		$index = 0;
		if (!is_array($needle))
			return $this->parse_strcasecmp($needle, $haystack, $offset);
		foreach ($needle as $item)
		{
			if ($this->parse_strcasecmp($item, $haystack, $offset))
				return true;
			++$index;
		}
		return false;
	}

	function parse_do_callback($callback, &$match, &$string, &$offset, &$length)
	{
		$match[DiscloseSecret_MATCH_ALL][0] = substr($string, $match[DiscloseSecret_MATCH_ALL][1],
			$offset - $match[DiscloseSecret_MATCH_ALL][1] + 1);
		$replace = call_user_func_array($callback, array($match));
		$length_diff = strlen($match[DiscloseSecret_MATCH_ALL][0]) - strlen($replace);
		$string = substr($string, 0, $match[DiscloseSecret_MATCH_ALL][1]). $replace. substr($string, $offset + 1);
		$offset -= $length_diff - 1;
		$length -= $length_diff;
	}

	function parse_quoted_string(&$match, &$string, &$offset, &$length, $item, $c_index)
	{
		$match[$c_index][1] = $offset;
		$match[$c_index + 1][0] = $item;
		$match[$c_index + 1][1] = $offset;
		$offset += strlen($item);
		$stop = DiscloseSecret_PARSE_NOT_FINISHED;
		while ($offset < $length && $stop == DiscloseSecret_PARSE_NOT_FINISHED)
		{
			// ((\'\")((?:[^\\2\\\]|\\\.)*?)\\2)
			//                             ^
			if ($this->parse_casecmp($match[$c_index + 1][0], $string, $offset, $index))
			{
				$match[$c_index][0] = substr($string, $match[$c_index + 1][1], $offset - $match[$c_index + 1][1] + 1);
				$match[$c_index + 2][0] = substr($string, $match[$c_index + 1][1] + 1, $offset - $match[$c_index + 1][1] - 1);
				$match[$c_index + 2][1] = $match[$c_index + 1][1] + strlen($match[$c_index + 1][0]);
				$stop = DiscloseSecret_PARSE_COMPLETE;
			}
			// ((\'\")((?:[^\\2\\\]|\\\.)*?)\\2)
			//                     ^
			else if ($this->parse_casecmp('\\', $string, $offset, $index))
				$offset += 2;
			// ((\'\")((?:[^\\2\\\]|\\\.)*?)\\2)
			//           ^
			else
				$offset += 1;
		}
		if ($stop == DiscloseSecret_PARSE_NOT_FINISHED)
			$stop = DiscloseSecret_PARSE_INCOMPLETE;
		return $stop;
	}

	function parse_column_in_condition(&$match, &$string, &$offset, &$length, $item)
	{
		$match[DiscloseSecret_MATCH_IN_OPERATOR][0] = $item;
		$match[DiscloseSecret_MATCH_IN_OPERATOR][1] = $offset;

		$offset += strlen($item);
		$spaces = 0;

		while ($offset < $length && ctype_space($string[$offset]))
		{
			++$offset;
			++$spaces;
		}
		// \s+in\s*(\([^\)]*?\))
		//         ^
		if (!$this->parse_casecmp('(', $string, $offset, $index))
			return DiscloseSecret_PARSE_INCOMPLETE;
		$item = $string[$offset];
		$offset += strlen($item);
		$match[DiscloseSecret_MATCH_IN_QUOTED_STRING][1] = $offset;
		$stop = DiscloseSecret_PARSE_NOT_FINISHED;
		while ($offset < $length && $stop == DiscloseSecret_PARSE_NOT_FINISHED)
		{
			// \s+in\s*(\([^\)]*?\))
			//                   ^
			if ($this->parse_casecmp(')', $string, $offset, $index))
			{
				$match[DiscloseSecret_MATCH_IN_QUOTED_STRING][0] = substr($string,
					$match[DiscloseSecret_MATCH_IN_QUOTED_STRING][1],
					$offset - $match[DiscloseSecret_MATCH_IN_QUOTED_STRING][1]);
				$stop = DiscloseSecret_PARSE_COMPLETE;
			}
			// \s+in\s*(\([^\)]*?\))
			//            ^
			else
				$offset += 1;
		}
		if ($stop == DiscloseSecret_PARSE_NOT_FINISHED)
			$stop = DiscloseSecret_PARSE_INCOMPLETE;
		return $stop;
	}

	function parse_column_equal_condition(&$match, &$string, &$offset, &$length, $item)
	{
		$match[DiscloseSecret_MATCH_EQUAL_OPERATOR][0] = $item;
		$match[DiscloseSecret_MATCH_EQUAL_OPERATOR][1] = $offset;

		$offset += strlen($item);
		$spaces = 0;

		while ($offset < $length && ctype_space($string[$offset]))
		{
			++$offset;
			++$spaces;
		}
		$stop = DiscloseSecret_PARSE_NOT_FINISHED;
		$operator = array('\'', '"');
		// \s*(?:=|!=|<>|<|<=|>|>=)\s*(([\'\"])(?:[^\\8\\\]|\\\.)*?\\8)
		//                             ^
		if ($this->parse_casecmp($operator, $string, $offset, $index))
			$stop = $this->parse_quoted_string($match, $string, $offset, $length,
				$operator[$index], DiscloseSecret_MATCH_EQUAL_QUOTED_STRING);
		if ($stop == DiscloseSecret_PARSE_NOT_FINISHED)
			$stop = DiscloseSecret_PARSE_INCOMPLETE;
		return $stop;
	}

	function parse_column_post_assertion(&$string, $offset, &$length)
	{
		$offset += 1;
		while ($offset < $length && ctype_space($string[$offset]))
			$offset += 1;
		if ($offset >= $length)
			return DiscloseSecret_PARSE_COMPLETE;
		if ($this->parse_casecmp(',', $string, $offset, $index))
			return DiscloseSecret_PARSE_INCOMPLETE;
		return DiscloseSecret_PARSE_COMPLETE;
	}

	function parse_column_pre_assertion(&$string, $offset)
	{
		$separator = 0;
		while ($offset > 0 && !ctype_alpha($string[$offset - 1]) && $string[$offset - 1] != '_')
		{
			++$separator;
			$offset -= 1;
		}
		$offset -= 3;
		if ($offset < 0)
			return DiscloseSecret_PARSE_COMPLETE;
		if (!$separator)
			return DiscloseSecret_PARSE_INCOMPLETE;
		if (!$this->parse_casecmp('SET', $string, $offset, $index))
			return DiscloseSecret_PARSE_COMPLETE;
		if ($offset <= 0)
			return DiscloseSecret_PARSE_INCOMPLETE;
		if (!ctype_alpha($string[$offset - 1]) && $string[$offset - 1] != '_')
			return DiscloseSecret_PARSE_INCOMPLETE;
		return DiscloseSecret_PARSE_COMPLETE;
	}

	function parse_column_prefix(&$match, &$string, $offset, &$length)
	{
		$back_offset = $offset;
		if ($back_offset <= 0)
			return DiscloseSecret_PARSE_COMPLETE;
		else if ($string[$back_offset - 1] == '.')
		{
			$back_offset -= 1;
			while ($back_offset > 0 && (ctype_alpha($string[$back_offset - 1]) || $string[$back_offset - 1] == '_'))
				$back_offset -= 1;
			if ($offset - $back_offset <= 1)
				return DiscloseSecret_PARSE_INCOMPLETE;
			$match[DiscloseSecret_MATCH_TABLE_PREFIX][0] = substr($string, $back_offset, $offset - $back_offset);
			$match[DiscloseSecret_MATCH_TABLE_PREFIX][1] = $back_offset;
			$match[DiscloseSecret_MATCH_ALL][1] = $back_offset;
		}
		else if (ctype_alpha($string[$back_offset - 1]) || $string[$back_offset - 1] == '_')
			return DiscloseSecret_PARSE_INCOMPLETE;
		return $this->parse_column_pre_assertion($string, $back_offset);
	}

	function parse_column_condition(&$match, &$string, &$offset, &$length, $item)
	{
		$match[DiscloseSecret_MATCH_COLUMN_NAME][0] = $item;
		$match[DiscloseSecret_MATCH_COLUMN_NAME][1] = $offset;
		$match[DiscloseSecret_MATCH_ALL][1] = $offset;

		$stop = $this->parse_column_prefix($match, $string, $offset, $length);
		if ($stop == DiscloseSecret_PARSE_INCOMPLETE)
			return $stop;

		$offset += strlen($item);
		$spaces = 0;
		while ($offset < $length && ctype_space($string[$offset]))
		{
			++$offset;
			++$spaces;
		}
		$stop = DiscloseSecret_PARSE_NOT_FINISHED;
		while ($offset < $length && $stop == DiscloseSecret_PARSE_NOT_FINISHED)
		{
			$operator = array('=', '!=', '<>', '<', '<=', '>', '>=');
			// \s*(?:=|!=|<>|<|<=|>|>=)\s*([\'\"])((?:[^\\8\\\]|\\\.)*?\\8)
			//    ^
			if ($this->parse_casecmp($operator, $string, $offset, $index))
				$stop = $this->parse_column_equal_condition($match, $string, $offset, $length, $operator[$index]);
			// \s+in\s*(\([^\)]*?\))
			//    ^
			else if ($this->parse_casecmp('in', $string, $offset, $index) && $spaces)
				$stop = $this->parse_column_in_condition($match, $string, $offset, $length, 'in');
			else if ($this->parse_casecmp('not', $string, $offset, $index) && $spaces)
			{
				while ($offset < $length && ctype_space($string[$offset]))
					++$offset;
				if ($this->parse_casecmp('in', $string, $offset, $index) && $spaces)
					$stop = $this->parse_column_in_condition($match, $string, $offset, $length, 'not in');
				else
					$stop = DiscloseSecret_PARSE_INCOMPLETE;
			}
			// no space or operator found; reset offset to next char
			else
				$stop = DiscloseSecret_PARSE_INCOMPLETE;
		}

		if ($stop == DiscloseSecret_PARSE_COMPLETE)
		{
			$stop = $this->parse_column_post_assertion($string, $offset, $length);
			$match[DiscloseSecret_MATCH_CONDITION][1] = isset($match[DiscloseSecret_MATCH_TABLE_PREFIX][1]) ?
				 $match[DiscloseSecret_MATCH_TABLE_PREFIX][1] : $match[DiscloseSecret_MATCH_COLUMN_NAME][1];
			$match[DiscloseSecret_MATCH_CONDITION][0] = substr($string, $match[DiscloseSecret_MATCH_CONDITION][1],
				$offset - $match[DiscloseSecret_MATCH_CONDITION][1] + 1);
		}
		if ($stop == DiscloseSecret_PARSE_NOT_FINISHED)
			$stop = DiscloseSecret_PARSE_INCOMPLETE;
		return $stop;
	}

	function parse($callback, $string)
	{
		// make some checks before parsing, speed things up a bit
		if (strpos($string, 'post_status') !== false || strpos($string, 'comment_approved') !== false)
		{
			$length = strlen($string);
			$offset = 0;

			while ($offset < $length)
			{
				$stop = DiscloseSecret_PARSE_NOT_FINISHED;
				$match = array();
				$operator_quote = array('\'', '"');
				$operator_column = array('post_status', 'comment_approved');
				// ((\'\")((?:[^\\2\\\]|\\\.)*?)\\2)
				// ^
				if ($this->parse_casecmp($operator_quote, $string, $offset, $index))
				{
					$stop = $this->parse_quoted_string($match, $string, $offset, $length,
						$operator_quote[$index], DiscloseSecret_MATCH_VALUE_QUOTED_STRING);
					if ($stop == DiscloseSecret_PARSE_COMPLETE)
					{
						$stop = DiscloseSecret_PARSE_SKIP_CALLBACK;
						$match[DiscloseSecret_MATCH_ALL][1] = $offset;
					}
				}
				// (?<!SET)?\s*(([\w-]+?\.)?(<column_name>)(?:\s*(?:=|!=|<>|<|<=|>|>=)\s*(([\'\"])(?:[^\\8\\\]|\\\.)*?\\8)|\s+in\s*(\([^\)]*?\))))\s*(?!,)?
				//                           ^
				else if ($this->parse_casecmp($operator_column, $string, $offset, $index))
					$stop = $this->parse_column_condition($match, $string, $offset, $length, $operator_column[$index]);

				if ($stop == DiscloseSecret_PARSE_COMPLETE)
					$this->parse_do_callback($callback, $match, $string, $offset, $length);
				else if ($stop == DiscloseSecret_PARSE_SKIP_CALLBACK)
					$offset = $match[DiscloseSecret_MATCH_ALL][1] + 1;
				else
					$offset += 1;
			}
		}
		return $string;
	}
}

?>