<?php

require_once(dirname(__FILE__).'/cache.php');
require_once(dirname(__FILE__).'/const.php');

// ----------------------------------------------------------------------------
// the DiscloseSecret_WriteUi class displays the restriction boxes on the
// Write admin panel
// ----------------------------------------------------------------------------

class DiscloseSecret_WriteUi
{
	var $m_cache;

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	// Sets up the Disclose-Secret admin menu
	function DiscloseSecret_WriteUi(&$cache, $activation_status)
	{
		$this->m_cache = $cache;
		if ($activation_status == DiscloseSecret_STATUS_OKAY)
			add_filter('admin_menu', array(&$this, 'filter_admin_menu_write'));
	}

	function filter_admin_menu_write()
	{
		add_filter('manage_posts_custom_column', array(&$this, 'filter_manage_posts_custom_column'));
		add_filter('manage_posts_columns', array(&$this, 'filter_manage_posts_columns'));
		add_filter('dbx_post_advanced', array(&$this, 'filter_dbx_edit_advanced'));
		add_filter('dbx_page_advanced', array(&$this, 'filter_dbx_edit_advanced'));
		add_filter('admin_head', array(&$this, 'filter_admin_head'));
	}

	function filter_admin_head()
	{
?>
	<style>
		#<?php echo DiscloseSecret_META_CAP; ?> label,
		#<?php echo DiscloseSecret_META_ROLE; ?> label,
		#<?php echo DiscloseSecret_META_USER; ?> label {
			float: left;
			width: 20em;
		}
	</style>

<?php
	}

	// ----------------------------------------------------------------------------
	// write interface
	// ----------------------------------------------------------------------------

	function print_restriction_form($restriction_options, $restrictions,
		$post_key, $meta_key, $caption, $help, $legend)
	{
?>
<div class="dbx-b-ox-wrapper">
	<fieldset id="<?php echo $meta_key; ?>" class="dbx-box">
		<div class="dbx-h-andle-wrapper">
			<h3 class="dbx-handle"><?php echo $caption; ?></h3>
		</div>
		<div class="dbx-c-ontent-wrapper">
			<div class="dbx-content">
				<p><?php echo $help; ?></p>
				<p><?php echo $legend; ?></p>
<?php
		foreach ($restriction_options as $id => $item)
		{
			$hint = '';
			if ($item[DiscloseSecret_INDEX_CAN_SEE_IMPLICIT])
				$hint .= ' [*]';
			if ($item[DiscloseSecret_INDEX_CAN_SEE_PROTECTED])
				$hint .= ' [!]';

			$mark = '';
			if (is_array($restrictions) && in_array($id, $restrictions))
			{
				$mark = '" checked="checked"';
			}

			$name = ' '. $item[DiscloseSecret_INDEX_NAME];
			if (!empty($hint) || !empty($mark))
			{
				$hint = '<strong>'. $hint;
				$name .= ' </strong>';
			}
			$mark .= '" />';
?>
				<label><input type="checkbox" name="<?php echo $post_key. $id; ?>" id="<?php echo $post_key. $id. $mark. $hint. $name; ?></label>
<?php
		}
?>
				<div style="clear: both;"></div>
			</div>
		</div>
	</fieldset>
</div>
<?php
	}

	function print_cap_form($option, $postmeta)
	{
		if (!$option->get_print_cap_form())
			return;
		$this->print_restriction_form($option->get_caps(), $postmeta->get_caps(),
			DiscloseSecret_POST_CAP, DiscloseSecret_META_CAP,
			__('Access by capability', DiscloseSecret_PLUGIN_ID),
			__('Mark one or more capability that should be used to restrict access to this article. Only users with at least one of the marked capabilities will be able to view this article and its comments. The article will automatically be set to private and will not be seen by the public if you restrict the access.', DiscloseSecret_PLUGIN_ID),
			__('<ul><li><input type="checkbox" checked="checked" readonly="readonly" /> Users having a capability with a marked checkbox will be able to read the protected article and its comments.</li>', DiscloseSecret_PLUGIN_ID) .
			__('<li><strong>[!]</strong> Users having a capability marked with an exclamation mark will <strong>always</strong> be able to read the protected or private article and its comments, regardless of the settings you will do on this form. This behaviour can not be changed due to WordPress.</li></ul>', DiscloseSecret_PLUGIN_ID));
	}

	function print_role_form($option, $postmeta)
	{
		if (!$option->get_print_role_form())
			return;
		$this->print_restriction_form($option->get_roles(), $postmeta->get_roles(),
			DiscloseSecret_POST_ROLE, DiscloseSecret_META_ROLE,
			__('Access by role', DiscloseSecret_PLUGIN_ID),
			__('Mark one or more role that should be used to restrict access to this article. Only users with one of the highlighted roles will be able to view this article and its comments. The article will automatically set to private and will not be seen by the public if you restrict the access.', DiscloseSecret_PLUGIN_ID),
			__('<ul><li><input type="checkbox" checked="checked" readonly="readonly" /> Users having a role with a marked checkbox will be able to read the protected article and its comments.</li>', DiscloseSecret_PLUGIN_ID).
			__('<li><strong>[!]</strong> Users having a role marked with an exclamation mark will <strong>always</strong> be able to read the protected or private article and its comments, regardless of the settings you will do on this form. This behaviour can not be changed due to WordPress.</li>', DiscloseSecret_PLUGIN_ID).
			__('<li><strong>[*]</strong> Users having a role marked with an asterisk will be able to read the protected article and its comments, because of settings you did for &quot;Access by capability&quot;.</li></ul>', DiscloseSecret_PLUGIN_ID));
	}

	function print_user_form($option, $postmeta)
	{
		if (!$option->get_print_user_form())
			return;
		$this->print_restriction_form($option->get_users(), $postmeta->get_users(),
			DiscloseSecret_POST_USER, DiscloseSecret_META_USER,
			__('Access by user', DiscloseSecret_PLUGIN_ID),
			__('Mark one or more user that should be able to view this article. The article will automatically set to private and will not be seen by the public if you restrict the access.', DiscloseSecret_PLUGIN_ID),
			__('<ul><li><input type="checkbox" checked="checked" readonly="readonly" /> Users with a marked checkbox will be able to read the protected article and its comments.</li>', DiscloseSecret_PLUGIN_ID).
			__('<li><strong>[!]</strong> Users marked with an exclamation mark will <strong>always</strong> be able to read the protected or private article and its comments, regardless of the settings you will do on this form. This behaviour can not be changed due to WordPress.</li>', DiscloseSecret_PLUGIN_ID).
			__('<li><strong>[*]</strong> Users marked with an asterisk will be able to read the protected article, because of settings you did for &quot;Access by capability&quot; or &quot;Access by role&quot;.</li></ul>', DiscloseSecret_PLUGIN_ID));
	}

	// Called during the edit form, outputs the Disclose-Secret drop down
	function filter_dbx_edit_advanced()
	{
		global $post;

		$id = 0;
		if (isset($post->ID))
			$id = $post->ID;
		$postmeta =& $this->m_cache->get_postmeta($id);
		$option =& $this->m_cache->get_option();
		$option->set_can_see_X($postmeta);
		$this->print_user_form($option, $postmeta);
		$this->print_role_form($option, $postmeta);
		$this->print_cap_form($option, $postmeta);
	}

	// ----------------------------------------------------------------------------
	// manage interface
	// ----------------------------------------------------------------------------

	// Adds an article restriction column to the manage post list
	function filter_manage_posts_columns($columns)
	{
		$columns[DiscloseSecret_PLUGIN_ID] = __('Status', DiscloseSecret_PLUGIN_ID);
		return $columns;
	}

	// Outputs the article restriction for each post in the manage post list
	function filter_manage_posts_custom_column($column_name)
	{
		if ($column_name != DiscloseSecret_PLUGIN_ID)
			return false;

		global $post;
		$postmeta =& $this->m_cache->get_postmeta($post->ID);
		if ($postmeta->is_private())
			_e('<strong>Private</strong>', DiscloseSecret_PLUGIN_ID);
		else if ($postmeta->is_protected())
			_e('<em>Protected</em>', DiscloseSecret_PLUGIN_ID);
		else
			_e('Public', DiscloseSecret_PLUGIN_ID);
		return true;
	}
}

?>